
#include <pspdebug.h>
#include <pspdisplay.h>
#include <pspiofilemgr.h>
#include <pspctrl.h>
#include <pspgu.h>
#include <string.h>
#include <stdio.h>

#include "shared.h"
#include "fileSelector.h"
#include "SMS.h"
#include "system.h"

extern int systemInit;

// Used to pass across the state to the other psp during wifi play
void sendStateData(void);
void recvStateData(void);

enum 
{
	LOAD_ROM=0,
	SCREEN_MODE,
	LOAD_GAME,
	SAVE_GAME,
	WIFI_MODE,
	SYSTEM_RESET,
	MAX_MENU
} MENU_SELECTIONS;


extern int g_Server;

const char selections[MAX_MENU][30] = { "Load ROM              \n\0",
					             "Screen Mode\0          ",
					             "Load Game State       \n\0",
					             "Save Game State       \n\0",
								 "Adhoc Wifi \0          ",
								 "SMS/GG Restart         "};

int pspLoadSystemState(int slot)
{
    char name[2048];
    FILE *fd;
    strcpy(name, SmsConfig.GameName);
    strcpy(strrchr(name, '.'), ".sav");
	
	fd = fopen(name, "rb");
	if(fd)
	{
		system_load_state(fd);

		fclose(fd);

		return 1;
	}

	return -1;
}

int pspSaveSystemState(int slot)
{
    char name[2048];
    FILE *fd;
    strcpy(name, SmsConfig.GameName);
    strcpy(strrchr(name, '.'), ".sav");

	fd = fopen(name, "wb");
	if(fd)
	{
		system_save_state(fd);
		fclose(fd);
	
		return 1;
	}

	return -1;
}

void displayMenu(void)
{
	int done = -1;
	int pos = 0;
	int i;
	int max=0;
	int update = 1;

	SceCtrlData m_PspPad, m_OldPad;
	pspDebugScreenInit();


	do
	{
		sceCtrlReadBufferPositive(&m_PspPad, 1);

		if(update)
		{
			pspDebugScreenSetXY(0,0);
			pspDebugScreenSetTextColor(0xFFFF);
			pspDebugScreenPrintf("\n\tSMSPlus 1.2\n");
			pspDebugScreenPrintf("\n\t-----------\n");
			pspDebugScreenPrintf("\tCopyright (C) Charles MacDonald 1998-2004\n");
			pspDebugScreenPrintf("\tPorted to the PSP by Chris Swindle (c_swindle@hotmail.com)\n");
			pspDebugScreenPrintf("\t\t- Build date: %s, %s\n\n", __DATE__, __TIME__);
			pspDebugScreenPrintf("\nPress X to select, triangle to return to game\n");

			pspDebugScreenSetXY(0,11);

			if(!systemInit)
				max = LOAD_GAME;
			else
				max = MAX_MENU;

			for(i=0; i<max; i++)
			{
				if(i==pos)
					pspDebugScreenSetTextColor(0xFF00);
				else
					pspDebugScreenSetTextColor(0xFFFF);
			
				pspDebugScreenPrintf("  %s", selections[i]);
				if(i==SCREEN_MODE)
				{
					pspDebugScreenPrintf(" - %s\n", SmsConfig.FullScreen ? "Fullscreen" : "Keep aspect ratio");
				}
				if(i==WIFI_MODE)
				{
					pspDebugScreenPrintf(" - %s\n", SmsConfig.WifiEnabled ? "Enabled" : "Disabled");
				}

			}
			update = 0;
		}

		if(m_PspPad.Buttons != m_OldPad.Buttons)
		{
			m_OldPad.Buttons = m_PspPad.Buttons;
			if(m_PspPad.Buttons & PSP_CTRL_UP)
			{
				if(pos > 0)
				{
					pos--;
					update = 1;
				}
			}

			if(m_PspPad.Buttons & PSP_CTRL_DOWN)
			{
				if(!systemInit)
					max = LOAD_GAME;
				else
					max = MAX_MENU;
				if(pos < max-1)
				{
					pos++;
					update = 1;
				}
			}

			if(m_PspPad.Buttons & PSP_CTRL_CROSS)
			{
				switch(pos)
				{
					case LOAD_ROM:
					{
						if(selectFile(SmsConfig.GameName)>0)
						{
							if(systemInit)
							{
								system_poweroff();
							}

							if(load_rom(SmsConfig.GameName) == 0) 
							{
								return;
							}

							if(systemInit)
							{
								sms_init();
								render_init();
								system_poweron();
							}
							done = 1;
						}
						
						break;
					}
					case LOAD_GAME:
					{
						if(systemInit)
						{
							pspLoadSystemState(1);
							done = 1;
						}
						break;
					}
					case SAVE_GAME:
					{
						if(systemInit)
						{
							pspSaveSystemState(1);
							done = 1;
						}
						break;
					}
					case SCREEN_MODE:
					{
						SmsConfig.FullScreen = !SmsConfig.FullScreen;
						done = 1;
						break;
					}
					case WIFI_MODE:
					{
						input.pad[0] = 0;
						input.pad[1] = 0;
						input.system = 0;

						if(SmsConfig.WifiEnabled)
						{
							adhocTerm();
							SmsConfig.WifiEnabled = 0;
						}
						else
						{
							pspDebugScreenInit();
							if((adhocInit(SmsConfig.GameName) >= 0) && (adhocSelect() >=0))
							{
								static char buffer[0x8000];
								unsigned int length;
								int err = 0;
								int i=0;
								SmsConfig.WifiEnabled = 1;
								SmsConfig.Server = g_Server;

								sceKernelDelayThread(3000000);

								system_poweroff();

								if(SmsConfig.Server)
								{
									sceKernelDelayThread(3000000);
									int size = cart.pages * 0x4000;

									// Now send the data to the other unit

									// send the rom to the client, first send the size
									pspDebugScreenPrintf("Sending ROM to the client\n");
									err = adhocSendRecvAck(&size, 4);
									err = adhocSendRecvAck(cart.rom, size);
									
									pspDebugScreenPrintf("Done sending data\n");
								}
								else
								{
									int size = 0;

									// receive the data
									pspDebugScreenPrintf("Waiting for data\n");

									// get the rom from the server
									length = 4;
									err = adhocRecvSendAck(&size, &length);

									cart.pages = (size / 0x4000);

									if(cart.rom)
									{
										free(cart.rom);
										cart.rom = NULL;
									}
									cart.rom = malloc(size);
									if(!cart.rom)
									{
										SmsConfig.WifiEnabled = 0;
										adhocTerm();
										return;
									}

									length = size;
									err = adhocRecvSendAck(cart.rom, &length);
									pspDebugScreenPrintf("Received ROM from server\n");

									SmsConfig.GameName[0] = '\0';
	
									/* Figure out game image type */
									sms.console = CONSOLE_SMS;

									system_assign_device(PORT_A, DEVICE_PAD2B);
									system_assign_device(PORT_B, DEVICE_PAD2B);
								}

								sms_init();
								render_init();
								system_poweron();
								done = 1;
							}
							else
							{
								adhocTerm();
								SmsConfig.WifiEnabled = 0;
							}
						}
					
						pspDebugScreenInit();
						update = 1;
						break;
					}
					case SYSTEM_RESET:
					{
						if(SmsConfig.GameName[0] != '\0' && !SmsConfig.WifiEnabled)
						{
							// Only reset if we are not playing wifi and we have
							// loaded a game
							system_reset();
						}
						done = 1;
						break;
					}
				}
			}

			if(m_PspPad.Buttons & PSP_CTRL_TRIANGLE)
				done = 1;

			m_OldPad.Buttons = m_PspPad.Buttons;
		}
		
		if(done && (SmsConfig.GameName[0] == '\0' && !(SmsConfig.WifiEnabled && !SmsConfig.Server)))
		{
			update = 1;
			done = -1;
		}

	} while(done<0);

	pspDebugScreenInit();

	VideoGuScreenClear();
}
