
#include <pspkernel.h>
#include <pspmodulemgr.h>
#include <pspdisplay.h>
#include <pspdebug.h>
#include <pspthreadman.h>
#include <pspctrl.h>
#include <pspsdk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "shared.h"
#include "SMS.h"
#include "system.h"

int systemInit = 0;

void SmsInit(void)
{

//	sceDisplayWaitVblankStart();
//	sceGuSwapBuffers();

	systemInit = 1;
}

void SmsTerm(void)
{
	if(systemInit)
	{
		// Close any network resources, the check is done in the network modules
		adhocTerm();

		SoundTerm();
		ControlsTerm();
		VideoGuTerm();

		system_poweroff();
		system_shutdown();
	}

	systemInit = 0;
}

void SmsEmulate(char *RomPath)
{
	int Quit=0;

	// Setup the 
	ChangeCurrentDir(RomPath);

	SmsConfig.FullScreen = 1;
	SmsConfig.SoundEnabled = 1;
	
	if(SmsConfig.SoundEnabled)
	{
		snd.sample_rate    = 44100;
		snd.fps            = 60;
		snd.psg_clock      = 3579545;
		snd.mixer_callback = NULL;
		sms.use_fm         = 0;
	}

	memset(&bitmap, 0, sizeof(bitmap_t));
	bitmap.width  = 256;
	bitmap.height = 192;
	bitmap.depth  = 8;
	bitmap.pitch  = 256;

	if(SmsConfig.SoundEnabled)
		SoundInit();

	ControlsInit();
	VideoGuInit();

	SmsConfig.GameName[0] = '\0';

	sms.save = 1;

	// Display the menu to load up the ROM or connect to an adhoc server
	displayMenu();

	system_init();

	system_poweron();

	systemInit = 1;

	while(!Quit)
	{
		ControlsUpdate();

		system_frame(0);

		VideoGuUpdate(SmsConfig.FullScreen);
		
		SoundUpdate();

		sceDisplayWaitVblankStart();
		sceGuSwapBuffers();
	}

	SmsTerm();
}

/* Save or load SRAM */
void system_manage_sram(uint8 *sram, int slot, int mode)
{
    char name[PATH_MAX];
    FILE *fd;

	if(SmsConfig.WifiEnabled)
	{
		memset(sram, 0x00, 0x8000);
		return;
	}

	if(SmsConfig.GameName[0] == '\0')
		return;

    strcpy(name, SmsConfig.GameName);
    strcpy(strrchr(name, '.'), ".sram");

    switch(mode)
    {
        case SRAM_SAVE:
            if(sms.save)
            {
                fd = fopen(name, "wb");
                if(fd)
                {
                    fwrite(sram, 0x8000, 1, fd);
                    fclose(fd);
                }
            }
            break;

        case SRAM_LOAD:
            fd = fopen(name, "rb");
            if(fd)
            {
                sms.save = 1;
                fread(sram, 0x8000, 1, fd);
                fclose(fd);
            }
            else
            {
                /* No SRAM file, so initialize memory */
                memset(sram, 0x00, 0x8000);
            }
            break;
    }
}

