
#include <pspdisplay.h>
#include <pspgu.h>
#include "shared.h"
#include "VideoGu.h"

uint32 pspGuList [262144] __attribute__((aligned(16)));
screen_t Screen;

struct Vertex
{
  unsigned short u, v;
  unsigned short color;
  short x, y, z;
};

void pspScaleImage(char *pixels, uint16 srcX, uint16 srcY, uint16 offsetX, 
				   uint16 offsetY, uint16 srcPitch, uint16 dstX, uint16 dstY);

void VideoGuInit(void)
{
	sceGuInit();

	sceGuStart(0, pspGuList);
	sceGuDrawBufferList(GU_PSM_5551, (void *)0, 512);
	sceGuDispBuffer(480, 272, (void *)0x88000, 512);
	sceGuDepthBuffer((void *)0x110000, 512);
	sceGuOffset(2048 - (480/2), 2048 - (272/2));
	sceGuViewport(2048, 2048, 480, 272);
	sceGuDepthRange(0xc350, 0x2710);
	sceGuScissor(0, 0, 480, 272);
	sceGuEnable(GU_SCISSOR_TEST);
	sceGuFrontFace(GU_CW);
	sceGuEnable(GU_TEXTURE_2D);
	sceGuClear(GU_COLOR_BUFFER_BIT | GU_DEPTH_BUFFER_BIT);
	sceGuFinish();

	sceGuSync(0, 0);
	sceDisplayWaitVblankStart();
	sceGuDisplay(1);

	bitmap.data   = (unsigned char*)Screen.scrBuffer;
}

void VideoGuTerm(void)
{
	sceGuTerm();
}

void VideoGuUpdate(int fullScreen)
{
	if(fullScreen)
	{
		pspScaleImage((char *)&Screen.scrBuffer, 
						bitmap.viewport.w,
						bitmap.viewport.h,
						bitmap.viewport.x,
						bitmap.viewport.y,
						256,
						480, 272);
	} 
	else
	{
		pspScaleImage((char *)&Screen.scrBuffer, 
							bitmap.viewport.w,
							bitmap.viewport.h,
							bitmap.viewport.x,
							bitmap.viewport.y,
							256,
							272*(bitmap.viewport.w/bitmap.viewport.h), 272);
	}
}


void VideoGuScreenClear(void)
{
	VideoGuTerm();
	VideoGuInit();
	sceGuStart (0, pspGuList);

	sceGuScissor(0, 0, 480, 272);
	sceGuEnable(GU_SCISSOR_TEST);
	sceGuClearColor(0x0);
	sceGuClear(GU_COLOR_BUFFER_BIT | GU_DEPTH_BUFFER_BIT);
	
	sceGuFinish ();
	sceGuSync(0, 0);

	sceGuSwapBuffers();
	sceGuStart (0, pspGuList);

	sceGuScissor(0, 0, 480, 272);
	sceGuEnable(GU_SCISSOR_TEST);
	sceGuClearColor(0x0);
	sceGuClear(GU_COLOR_BUFFER_BIT | GU_DEPTH_BUFFER_BIT);
	
	sceGuFinish ();
	sceGuSync(0, 0);
}


#define SLICE_SIZE 64 // Change this to experiment with different page-cache sizes

void pspScaleImage(char *pixels, uint16 srcX, uint16 srcY, uint16 offsetX, 
				   uint16 offsetY, uint16 srcPitch, uint16 dstX, uint16 dstY)
{
	uint16 x, i, j;
	uint8 r, g, b;
	struct Vertex* vertices;
	const int slice_scale = (int)(((float)dstX / (float)srcX) * 
				(float)SLICE_SIZE);

	int dstOffset = (480 - dstX)/2;

	if(bitmap.pal.update)
	{
		for(i=0; i<PALETTE_SIZE; i++)
		{
			r = bitmap.pal.color[i][0];
			g = bitmap.pal.color[i][1];
			b = bitmap.pal.color[i][2];

			Screen.pal[i] = MAKE_PIXEL(r,g,b);
			Screen.pal[i+PALETTE_SIZE] = MAKE_PIXEL(r,g,b);
			Screen.pal[i+(2*PALETTE_SIZE)] = MAKE_PIXEL(r,g,b);
			Screen.pal[i+(3*PALETTE_SIZE)] = MAKE_PIXEL(r,g,b);
		}
	}

	sceKernelDcacheWritebackAll();

	sceGuStart(GU_DIRECT,pspGuList);
	sceGuAmbientColor(0xffffffff);
	sceGuScissor(dstOffset, 0, dstX, dstY);
	sceGuClutMode(GU_PSM_5551,0,0xff,0);
	sceGuClutLoad((256/8),Screen.pal);
	sceGuTexMode(GU_PSM_T8,0,0,0);
	sceGuTexImage(0,256,256,srcPitch,pixels);
	sceGuTexFunc(GU_TFX_REPLACE,GU_TCC_RGB);
	sceGuTexFilter(GU_LINEAR,GU_LINEAR);
	sceGuTexScale(1,1); // scale UVs to 0..1
	sceGuTexOffset(0.0f,0.0f);

	// do a striped blit (takes the page-cache into account)
	x=dstOffset;

	for (j = offsetX; j < srcX+offsetX; j = j+SLICE_SIZE, x+= slice_scale)
	{
		vertices = (struct Vertex*)sceGuGetMemory(2 * sizeof(struct Vertex));
		vertices[0].u = j; vertices[0].v = offsetY;
		vertices[0].color = 0;
		vertices[0].x = x; vertices[0].y = 0; vertices[0].z = 0;
		vertices[1].u = j+SLICE_SIZE; vertices[1].v = srcY+offsetY;
		vertices[1].color = 0;
		vertices[1].x = (x+slice_scale); vertices[1].y = dstY; 
		vertices[1].z = 0;

		sceGuDrawArray(GU_SPRITES,GU_TEXTURE_16BIT|GU_COLOR_5551|GU_VERTEX_16BIT|GU_TRANSFORM_2D,2,0,vertices);
	}

	sceGuFinish();
	sceGuSync(0,0);

	return;
}
