
#include <pspdebug.h>
#include <pspdisplay.h>
#include <pspiofilemgr.h>
#include <pspctrl.h>
#include <string.h>

#include <string>
#include <algorithm>
#include <vector>

using namespace std;

#define NUM_FILES 32

struct DirEntry
{
	string m_dirEntry;
	int   m_dir;

	DirEntry(string file, int dir)
	{
		m_dirEntry = file;
		m_dir = dir;
	}

	bool operator > (const DirEntry &dirEntry) const
	{
		if(m_dir && !dirEntry.m_dir)
			return false;

		if(!m_dir && dirEntry.m_dir)
			return true;

		return (m_dirEntry > dirEntry.m_dirEntry);
	}

	bool operator < (const DirEntry &dirEntry) const
	{
		if(m_dir && !dirEntry.m_dir)
			return true;

		if(!m_dir && dirEntry.m_dir)
			return false;

		return (m_dirEntry < dirEntry.m_dirEntry);
	}
};

typedef	vector<DirEntry> DirList;

class FileSelector
{
private:
	DirList m_DirList;
	char m_CurrentDir[1000];
	int GetDirList();
	int DisplayDirList();
	int CheckButtons();
	int ChangeCurrentDir();

	SceCtrlData m_PspPad;
	SceCtrlData m_OldPad;
	int m_Repeat;
	int m_Pos;
	int m_Max;

public:
	int SetCurrentDir(char *dir);
	int SelectFile(char *file);
	FileSelector();
	int m_Quit;
};

FileSelector::FileSelector(void)
{

}

int FileSelector::ChangeCurrentDir(void)
{
	char *temp;

	if(m_DirList[m_Pos].m_dir == 1)
	{
		if(strncmp(m_DirList[m_Pos].m_dirEntry.c_str(), "..", 2) == 0)
		{
			// Strip off the last directory
			temp = strrchr(m_CurrentDir-1, '/');

			if(temp > 0)
				*temp = '\0';
		}
		else if(strncmp(m_DirList[m_Pos].m_dirEntry.c_str(), ".", 1) == 0)
		{
			// Do nothing, just cause a refresh
		}
		else
		{
			strcat(m_CurrentDir, "/");
			strcat(m_CurrentDir, m_DirList[m_Pos].m_dirEntry.c_str());
		}

		return 1;
	}

	return -1;
}

SceIoDirent dirEntry;

int FileSelector::GetDirList(void)
{
	m_DirList.clear();

	int dirFd = sceIoDopen(m_CurrentDir);
	if(dirFd < 0)
	{
		return -1;
	}

	m_Max = 0;

	dirEntry.d_name[0] = '\0';

	while(sceIoDread(dirFd, &dirEntry) > 0)
	{
		int isDir = 0;

		if((dirEntry.d_stat.st_mode & 0xF000) == 0x1000)
		{
			isDir = 1;
		}

		m_DirList.push_back(DirEntry(dirEntry.d_name, isDir));	

		dirEntry.d_name[0] = '\0';
		dirEntry.d_stat.st_mode = 0;

		m_Max++;
	}

	sceIoDclose(dirFd);

	sort(m_DirList.begin(), m_DirList.end());

	m_Pos = 0;

	return 1;
}

int FileSelector::DisplayDirList()
{
	int i=0;
	int startPos, endPos;

	pspDebugScreenSetXY(0,0);
	pspDebugScreenPrintf("                                                                                           ");
	pspDebugScreenSetXY(0,0);

	if(m_Pos < (NUM_FILES/2))
	{
		startPos = 0;
	}
	else if(m_Pos > (m_Max - NUM_FILES/2))
	{
		startPos = m_Max - NUM_FILES;
	}
	else
	{
		startPos = m_Pos - (NUM_FILES/2);
	}

	if(startPos < 0)
		startPos = 0;

	endPos = min(m_Max, startPos + NUM_FILES);

	for(i=startPos; i<endPos; i++)
	{
		if(i == m_Pos)
		{
			pspDebugScreenSetTextColor(0xFF00);
		}
		else
		{
			pspDebugScreenSetTextColor(0xFFFF);
		}
		pspDebugScreenPrintf("  %s\n", m_DirList[i].m_dirEntry.c_str());
	}
}

int FileSelector::CheckButtons(void)
{
	sceCtrlReadBufferPositive(&m_PspPad, 1);

	if(!m_PspPad.Buttons)
		m_Repeat = 0;

	if(m_PspPad.Buttons == m_OldPad.Buttons)
	{
		m_Repeat++;
		if(m_Repeat > 0x20)
		{
			m_OldPad.Buttons = 0;
		}
	}

	if(m_PspPad.Buttons != m_OldPad.Buttons)
	{
		m_OldPad.Buttons = m_PspPad.Buttons;
		if(m_PspPad.Buttons & PSP_CTRL_UP)
		{
			if(m_Pos > 0)
			{
				m_Pos--;
				return 1;
			}
		}

		if(m_PspPad.Buttons & PSP_CTRL_DOWN)
		{
			if(m_Pos < m_Max-1)
			{
				m_Pos++;
				return 1;
			}
		}

		if(m_PspPad.Buttons & PSP_CTRL_CROSS)
			return -1;

		if(m_PspPad.Buttons & PSP_CTRL_TRIANGLE)
			return -2;
	}

	m_OldPad.Buttons = m_PspPad.Buttons;

	return 0;
}

int FileSelector::SetCurrentDir(char *dir)
{
	strcpy(m_CurrentDir,dir);

	GetDirList();
}


// Display a simple text window and print out the list of files with the
// extension specified
int FileSelector::SelectFile(char *file)
{
	int quit = 0;
	int rc = 0;
	int repeat = 0;
	int oldpos = 0;
	m_Quit = -1;

	pspDebugScreenInit();
	sceCtrlSetSamplingCycle(1);

	DisplayDirList();

	do
	{		
		sceDisplayWaitVblankStart();
		rc = CheckButtons();

		if(rc > 0)
			DisplayDirList();

		if(rc == -2)
		{
			return -1;
		}
		else if(rc < 0)
		{
			pspDebugScreenInit();
			if(ChangeCurrentDir()>0)
			{
				// Need to get the dir
				GetDirList();
				DisplayDirList();
			}
			else
			{
				// Copy the path and file into file
				strcpy(file, m_CurrentDir);
				strcat(file, "/");
				strcat(file, m_DirList[m_Pos].m_dirEntry.c_str());
				return 1;
			}
		}
	} while (!quit);
}

FileSelector fileSel;

extern "C" int selectFile(char *file)
{
//	FileSelector fileSel(dir, file);

	if(fileSel.SelectFile(file) < 0)
	{
		return -1;
	}

	return 1;
}

extern "C" int ChangeCurrentDir(char *dir)
{
	fileSel.SetCurrentDir(dir);
}
