#include <pspkernel.h>
#include <pspmodulemgr.h>
#include <pspdisplay.h>
#include <pspdebug.h>
#include <pspthreadman.h>
#include <pspctrl.h>
#include <pspsdk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "shared.h"

PSP_MODULE_INFO("SMSPlusWifi", 0x1000, 0, 1); /* 0x1000 REQUIRED!!! */
PSP_MAIN_THREAD_ATTR(0);

char                  RomPath [2048];
char                  file[1024];

extern char game_name[PATH_MAX];

/* Exit callback */
int exit_callback(void)
{
	SmsTerm();

	adhocTerm();

	sceKernelExitGame();

	return 0;
}

/* Callback thread */
int CallbackThread(SceSize args, void *argp)
{
	int cbid;

	cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
	sceKernelRegisterExitCallback(cbid);

	sceKernelSleepThreadCB();

	return 0;
}

// Copying the exception handler from SDLSMS as it gives a nice output to find the bug
void sdl_psp_exception_handler(PspDebugRegBlock *regs)
{
	pspDebugScreenInit();

	pspDebugScreenSetBackColor(0x00FF0000);
	pspDebugScreenSetTextColor(0xFFFFFFFF);
	pspDebugScreenClear();

	pspDebugScreenPrintf("I regret to inform you your psp has just crashed\n\n");
	pspDebugScreenPrintf("Exception Details:\n");
	pspDebugDumpException(regs);
	pspDebugScreenPrintf("\nThe offending routine may be identified with:\n\n"
		"\tpsp-addr2line -e target.elf -f -C 0x%x 0x%x 0x%x\n",
		regs->epc, regs->badvaddr, regs->r[31]);
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void)
{
	int thid = 0;

	thid = sceKernelCreateThread("update_thread", CallbackThread, 0x11, 0xFA0, 0, 0);
	if(thid >= 0)
	{
		sceKernelStartThread(thid, 0, 0);
	}

	return thid;
}

extern char *__psp_argv_0;
int user_main(void)
{
	SetupCallbacks();

	char *p;
	p = strrchr (RomPath, '/');
	*p = '\0';

	SmsEmulate(RomPath);

	sceKernelExitGame();
}

int main(int argc, char *argp[])
{
    // Kernel mode thread

    if (adhocLoadDrivers(&module_info) != 0)
    {
        printf("Driver load error\n");
        return 0;
    }

	// Need to copy this across otherwise the address is in kernel mode
	strcpy (RomPath, (char*)argp[0]);
	__psp_argv_0 = RomPath;

	pspDebugInstallErrorHandler(sdl_psp_exception_handler);

    // create user thread, tweek stack size here if necessary
    SceUID thid = sceKernelCreateThread("User Mode Thread", user_main,
            0x11, // default priority
            256 * 1024, // stack size (256KB is regular default)
            PSP_THREAD_ATTR_USER, NULL);

    // start user thread, then wait for it to do everything else
    sceKernelStartThread(thid, 0, 0);
    sceKernelWaitThreadEnd(thid, NULL);

    return 0;
}
